@@ -1,21 +1,19 @@
 require File.expand_path(File.join(File.dirname(__FILE__), 'helper'))
-require 'webrick'
-class ::WEBrick::HTTPServer ; def access_log(config, req, res) ; end ; end
-class ::WEBrick::BasicLog ; def log(level, data) ; end ; end
 
 class BugTestInstancePostDiffersFromClassPost < Test::Unit::TestCase
-  def test_bug
-    server = WEBrick::HTTPServer.new( :Port => 9999 )
-    server.mount_proc("/test") do|req,res|
+  include BugTestServerSetupTeardown
+
+  def setup
+    @port = 9999
+    @response_proc = lambda do|res|
       sleep 0.5
       res.body = "hi"
       res['Content-Type'] = "text/html"
     end
+    super
+  end
 
-    thread = Thread.new(server) do|srv|
-      srv.start
-    end
-
+  def test_bug
     threads = []
     timer = Time.now
 
@@ -45,8 +43,5 @@ class BugTestInstancePostDiffersFromClassPost < Test::Unit::TestCase
     puts "requested in #{single_time}"
 
     assert single_time > multi_time
-
-    server.shutdown
-    thread.join
   end
 end
