@@ -18,6 +18,20 @@ class TestCurbCurlEasy < Test::Unit::TestCase
     assert_equal 200, easy.code
   end
 
+  def test_curlopt_resolve
+    require 'resolv'
+    uri = URI.parse(TestServlet.url)
+    resolved_ip = Resolv.getaddress(uri.host)   # perform DNS lookup once
+    mapping = "#{uri.host}:#{uri.port}:#{resolved_ip}"
+
+    http = Curl::Easy.new(TestServlet.url)
+    http.setopt(Curl::CURLOPT_RESOLVE, [mapping])
+
+    http.get
+
+    assert_match(/GET/, http.body)
+  end
+
   def test_curlopt_stderr_with_file
     # does not work with Tempfile directly
     path = Tempfile.new('curb_test_curlopt_stderr').path
@@ -34,20 +48,62 @@ class TestCurbCurlEasy < Test::Unit::TestCase
   end
 
   def test_curlopt_stderr_with_io
-    path = Tempfile.new('curb_test_curlopt_stderr').path
+    tmp = Tempfile.new('curb_test_curlopt_stderr')
+    path = tmp.path
     fd = IO.sysopen(path, 'w')
-    io = IO.for_fd(fd)
+    io = IO.new(fd, 'w')
+    io.sync = true
 
     easy = Curl::Easy.new(TestServlet.url)
     easy.verbose = true
     easy.setopt(Curl::CURLOPT_STDERR, io)
     easy.perform
 
-
+    io.flush
+    io.close
     output = File.read(path)
 
-    assert_match(output, 'HTTP/1.1 200 OK')
-    assert_match(output, 'Host: 127.0.0.1:9129')
+    assert_match(/HTTP\/1\.1\ 200\ OK(?:\ )?/, output)
+    assert_match('Host: 127.0.0.1:9129', output)
+  ensure
+    tmp.close! if defined?(tmp) && tmp
+  end
+
+  def test_curlopt_stderr_gc_safe
+    tmp = Tempfile.new('curb_test_curlopt_stderr_gc')
+    path = tmp.path
+    fd = IO.sysopen(path, 'w')
+    io = IO.new(fd, 'w')
+
+    easy = Curl::Easy.new(TestServlet.url)
+    easy.verbose = true
+    easy.setopt(Curl::CURLOPT_STDERR, io)
+
+    # Drop our Ruby reference and force GC; curb should retain it internally
+    io = nil
+    GC.start
+
+    easy.perform
+
+    output = File.read(path)
+    assert_match(/HTTP\/1\.1\ 200\ OK(?:\ )?/, output)
+    assert_match('Host: 127.0.0.1:9129', output)
+  ensure
+    tmp.close! if defined?(tmp) && tmp
+  end
+
+  def test_head_request_no_body_and_no_timeout
+    # Ensure a HEAD request completes and does not attempt to read a body
+    # even when the server advertises a Content-Length.
+    easy = nil
+    assert_nothing_raised do
+      easy = Curl.http(:HEAD, TestServlet.url)
+    end
+    assert_not_nil easy
+    # Header string should contain the HTTP status line.
+    assert_match(/HTTP\/1\.1\s\d+\s/, easy.header_str.to_s)
+    # Body should be empty for HEAD requests (libcurl won't call the write callback).
+    assert_equal "", easy.body_str.to_s
   end
 
   def test_curlopt_stderr_fails_with_tempdir
@@ -110,7 +166,7 @@ class TestCurbCurlEasy < Test::Unit::TestCase
   end    
 
   def test_class_perform_02
-    data = ""
+    data = String.new
     assert_instance_of Curl::Easy, c = Curl::Easy.perform($TEST_URL) { |curl| curl.on_body { |d| data << d; d.length } }    
 
     assert_nil c.body_str
@@ -211,7 +267,7 @@ class TestCurbCurlEasy < Test::Unit::TestCase
   end    
 
   def test_get_02
-    data = ""
+    data = String.new
     c = Curl::Easy.new($TEST_URL) do |curl|
       curl.on_body { |d| data << d; d.length }
     end
@@ -231,6 +287,7 @@ class TestCurbCurlEasy < Test::Unit::TestCase
 
 
   def test_last_effective_url_01
+    omit('Windows file URL semantics differ') if WINDOWS
     c = Curl::Easy.new($TEST_URL)
     
     assert_equal $TEST_URL, c.url
@@ -765,20 +822,27 @@ class TestCurbCurlEasy < Test::Unit::TestCase
     assert_equal 'GET', curl.body_str
   end
   
-  def test_post_remote
+  def test_remote
     curl = Curl::Easy.new(TestServlet.url)
     curl.http_post([Curl::PostField.content('document_id', 5)])
     assert_equal "POST\ndocument_id=5", curl.unescape(curl.body_str)
+
+    curl.http_put([Curl::PostField.content('document_id', 5)])
+    assert_equal "PUT\ndocument_id=5", curl.unescape(curl.body_str)
+
+    curl.http_patch([Curl::PostField.content('document_id', 5)])
+    assert_equal "PATCH\ndocument_id=5", curl.unescape(curl.body_str)
   end
 
-  def test_post_remote_is_easy_handle
+  def test_remote_is_easy_handle
     # see: http://pastie.org/560852 and
     # http://groups.google.com/group/curb---ruby-libcurl-bindings/browse_thread/thread/216bb2d9b037f347?hl=en
-    [:post, :get, :head, :delete].each do |method|
+    [:post, :patch, :put, :get, :head, :delete].each do |method|
       retries = 0
       begin
         count = 0
-        Curl::Easy.send("http_#{method}", TestServlet.url) do|c|
+        m = "http_#{method}".to_sym
+        Curl::Easy.send(m, TestServlet.url) do|c|
           count += 1
           assert_equal Curl::Easy, c.class
         end
@@ -787,24 +851,40 @@ class TestCurbCurlEasy < Test::Unit::TestCase
         retries+=1
         retry if retries < 3
         raise e
+      rescue ArgumentError => e
+        assert false, "#{m} - #{e.message}"
       end
     end
   end
 
   # see: https://github.com/rvanlieshout/curb/commit/8bcdefddc0162484681ebd1a92d52a642666a445
-  def test_post_multipart_array_remote
+  def test_multipart_array_remote
     curl = Curl::Easy.new(TestServlet.url)
     curl.multipart_form_post = true
     fields = [
-      Curl::PostField.file('foo', File.expand_path(File.join(File.dirname(__FILE__),'..','README.markdown'))),
-      Curl::PostField.file('bar', File.expand_path(File.join(File.dirname(__FILE__),'..','README.markdown')))
+      Curl::PostField.file('foo', File.expand_path(File.join(File.dirname(__FILE__),'..','README.md'))),
+      Curl::PostField.file('bar', File.expand_path(File.join(File.dirname(__FILE__),'..','README.md')))
     ]
     curl.http_post(fields)
     assert_match(/HTTP POST file upload/, curl.body_str)
     assert_match(/Content-Disposition: form-data/, curl.body_str)
+
+    curl = Curl::Easy.new(TestServlet.url)
+    curl.multipart_form_post = true
+    fields = [
+      Curl::PostField.file('foo', File.expand_path(File.join(File.dirname(__FILE__),'..','README.md'))),
+      Curl::PostField.file('bar', File.expand_path(File.join(File.dirname(__FILE__),'..','README.md')))
+    ]
+    curl.http_put(fields)
+    assert_match(/HTTP POST file upload/, curl.body_str)
+    assert_match(/Content-Disposition: form-data/, curl.body_str)
+
+    curl.http_patch(fields)
+    assert_match(/HTTP POST file upload/, curl.body_str)
+    assert_match(/Content-Disposition: form-data/, curl.body_str)
   end
   
-  def test_post_with_body_remote
+  def test_with_body_remote
     curl = Curl::Easy.new(TestServlet.url)
     curl.post_body = 'foo=bar&encoded%20string=val'
     
@@ -812,23 +892,42 @@ class TestCurbCurlEasy < Test::Unit::TestCase
     
     assert_equal "POST\nfoo=bar&encoded%20string=val", curl.body_str
     assert_equal 'foo=bar&encoded%20string=val', curl.post_body
+
+    curl = Curl::Easy.new(TestServlet.url)
+    curl.put_data = 'foo=bar&encoded%20string=val'
+    
+    curl.perform
+    
+    assert_equal "PUT\nfoo=bar&encoded%20string=val", curl.body_str
   end
   
-  def test_form_post_body_remote
+  def test_form_body_remote
     curl = Curl::Easy.new(TestServlet.url)
     curl.http_post('foo=bar', 'encoded%20string=val')
     
     assert_equal "POST\nfoo=bar&encoded%20string=val", curl.body_str
     assert_equal 'foo=bar&encoded%20string=val', curl.post_body
+
+    curl = Curl::Easy.new(TestServlet.url)
+    curl.http_put('foo=bar', 'encoded%20string=val')
+    
+    assert_equal "PUT\nfoo=bar&encoded%20string=val", curl.body_str
+
+    curl = Curl::Easy.new(TestServlet.url)
+    curl.http_patch('foo=bar', 'encoded%20string=val')
+    
+    assert_equal "PATCH\nfoo=bar&encoded%20string=val", curl.body_str
   end
 
-  def test_post_multipart_file_remote
-    curl = Curl::Easy.new(TestServlet.url)
-    curl.multipart_form_post = true
-    pf = Curl::PostField.file('readme', File.expand_path(File.join(File.dirname(__FILE__),'..','README.markdown')))
-    curl.http_post(pf)
-    assert_match(/HTTP POST file upload/, curl.body_str)
-    assert_match(/Content-Disposition: form-data/, curl.body_str)
+  def test_multipart_file_remote
+    [:put, :post, :patch].each {|method|
+      curl = Curl::Easy.new(TestServlet.url)
+      curl.multipart_form_post = true
+      pf = Curl::PostField.file('readme', File.expand_path(File.join(File.dirname(__FILE__),'..','README.md')))
+      curl.send("http_#{method}", pf)
+      assert_match(/HTTP POST file upload/, curl.body_str)
+      assert_match(/Content-Disposition: form-data/, curl.body_str)
+    }
   end
 
   def test_delete_remote
@@ -985,7 +1084,11 @@ class TestCurbCurlEasy < Test::Unit::TestCase
     $auth_header = nil
     # curl checks the auth type supported by the server, so we have to create a 
     # new easy handle if we're going to change the auth type...
-
+    if WINDOWS
+      # On Windows, libcurl often uses SSPI for NTLM which yields a different
+      # header value and encoding; skip the NTLM-specific assertion.
+      return
+    end
     curl = Curl::Easy.new(TestServlet.url)
     curl.username = "foo"
     curl.password = "bar"
@@ -1006,7 +1109,7 @@ class TestCurbCurlEasy < Test::Unit::TestCase
   end
 
   def test_post_streaming
-    readme = File.expand_path(File.join(File.dirname(__FILE__),'..','README.markdown'))
+    readme = File.expand_path(File.join(File.dirname(__FILE__),'..','README.md'))
     
     pf = Curl::PostField.file("filename", readme)
 
@@ -1016,11 +1119,10 @@ class TestCurbCurlEasy < Test::Unit::TestCase
     easy.multipart_form_post = true
     easy.http_post(pf)
 
-    assert_not_equal(0,easy.body_str.size)
-    assert_equal(easy.body_str.tr("\r", ''), File.read(readme))
+    assert_not_equal(0,easy.body.size)
+    assert_equal(Digest::MD5.hexdigest(easy.body), Digest::MD5.hexdigest(File.binread(readme)))
   end
 
-
   def test_easy_close
     easy = Curl::Easy.new
     easy.close
@@ -1040,6 +1142,18 @@ class TestCurbCurlEasy < Test::Unit::TestCase
     easy.http_get
   end
 
+  def test_last_result_initialization
+    # Test for issue #463 - ensure last_result is properly initialized to 0
+    easy = Curl::Easy.new
+    assert_equal 0, easy.last_result, "last_result should be initialized to 0 for new instance"
+    
+    # Test that reset also sets last_result to 0
+    easy.url = TestServlet.url
+    easy.http_get
+    easy.reset
+    assert_equal 0, easy.last_result, "last_result should be reset to 0 after calling reset"
+  end
+
   def test_easy_use_http_versions
     easy = Curl::Easy.new
     easy.url = TestServlet.url + "?query=foo"
@@ -1065,16 +1179,22 @@ class TestCurbCurlEasy < Test::Unit::TestCase
     assert_equal "PUT\nhello", curl.body_str
     curl.http('COPY')
     assert_equal 'COPY', curl.body_str
+
+    curl.http_patch
+    assert_equal "PATCH\n", curl.body_str
+
+    curl.http_put
+    assert_equal "PUT\n", curl.body_str
   end
 
   def test_easy_http_verbs_must_respond_to_str
-    # issue http://github.com/taf2/curb/issues#issue/45
+    # issue http://github.com/taf2/curb/issues/45
     assert_nothing_raised do
-      c = Curl::Easy.new ; c.url = 'http://example.com' ; c.http(:get)
+      c = Curl::Easy.new ; c.url = TestServlet.url ; c.http(:get)
     end
 
     assert_raise RuntimeError do
-      c = Curl::Easy.new ; c.url = 'http://example.com' ; c.http(FooNoToS.new)
+      c = Curl::Easy.new ; c.url = TestServlet.url ; c.http(FooNoToS.new)
     end
 
   end
