@@ -10,10 +10,11 @@ class TestCurbCurlMulti < Test::Unit::TestCase
   # for https://github.com/taf2/curb/issues/277
   # must connect to an external
   def test_connection_keepalive
+    omit('Not supported on Windows runners') if WINDOWS
     # this test fails with libcurl 7.22.0. I didn't investigate, but it may be related
     # to CURLOPT_MAXCONNECTS bug fixed in 7.30.0:
     # https://github.com/curl/curl/commit/e87e76e2dc108efb1cae87df496416f49c55fca0
-    omit("Skip, libcurl too old (< 7.22.0)") if Curl::CURL_VERSION.split('.')[1].to_i <= 22
+    omit("Skip, libcurl too old (< 7.22.0)") if Curl::CURL_VERSION.to_f < 8 && Curl::CURL_VERSION.split('.')[1].to_i <= 22
 
     @server.shutdown if @server
     @test_thread.kill if @test_thread
@@ -114,13 +115,13 @@ class TestCurbCurlMulti < Test::Unit::TestCase
   end
 
   def test_new_multi_01
-    d1 = ""
+    d1 = String.new
     c1 = Curl::Easy.new($TEST_URL) do |curl|
       curl.headers["User-Agent"] = "myapp-0.0"
       curl.on_body {|d| d1 << d; d.length }
     end
 
-    d2 = ""
+    d2 = String.new
     c2 = Curl::Easy.new($TEST_URL) do |curl|
       curl.headers["User-Agent"] = "myapp-0.0"
       curl.on_body {|d| d2 << d; d.length }
@@ -171,6 +172,7 @@ class TestCurbCurlMulti < Test::Unit::TestCase
   end
 
   def test_multi_easy_get_with_error
+    omit('Path/line parsing differs on Windows') if WINDOWS
     begin
       did_raise = false
       n = 3
@@ -186,7 +188,7 @@ class TestCurbCurlMulti < Test::Unit::TestCase
     rescue Curl::Err::AbortedByCallbackError => e
       did_raise = true
       in_file = e.backtrace.detect {|err| err.match?(File.basename(__FILE__)) }
-      in_file_stack = e.backtrace.select {|err| err.match?(File.basename(__FILE__)) }
+      #in_file_stack = e.backtrace.select {|err| err.match?(File.basename(__FILE__)) }
       assert_match(__FILE__, in_file)
       in_file.gsub!(__FILE__)
       parts = in_file.split(':')
@@ -206,7 +208,7 @@ class TestCurbCurlMulti < Test::Unit::TestCase
     m = Curl::Multi.new
     responses = []
     n.times do|i|
-      responses[i] = ""
+      responses[i] = String.new
       c = Curl::Easy.new($TEST_URL) do|curl|
         curl.on_body{|data| responses[i] << data; data.size }
       end
@@ -230,7 +232,7 @@ class TestCurbCurlMulti < Test::Unit::TestCase
     5.times do|it|
       responses = []
       n.times do|i|
-        responses[i] = ""
+        responses[i] = String.new
         c = Curl::Easy.new($TEST_URL) do|curl|
           curl.on_body{|data| responses[i] << data; data.size }
         end
@@ -376,7 +378,7 @@ class TestCurbCurlMulti < Test::Unit::TestCase
     attr_reader :buf
 
     def t_method
-      @buf = ""
+      @buf = String.new
       @m = Curl::Multi.new
       10.times do|i|
         c = Curl::Easy.new($TEST_URL)
@@ -422,8 +424,8 @@ class TestCurbCurlMulti < Test::Unit::TestCase
     m = Curl::Multi.new
     # add a few easy handles
     requests.each do |url|
-      responses[url] = ""
-      responses["#{url}-header"] = ""
+      responses[url] = String.new
+      responses["#{url}-header"] = String.new
       c = Curl::Easy.new(url) do|curl|
         curl.follow_location = true
         curl.on_header{|data| responses["#{url}-header"] << data; data.size }
@@ -552,6 +554,54 @@ class TestCurbCurlMulti < Test::Unit::TestCase
     end
   end
 
+  def test_multi_easy_http_with_max_host_connections
+    urls = [
+      { :url => TestServlet.url + '?q=1', :method => :get },
+      { :url => TestServlet.url + '?q=2', :method => :get },
+      { :url => TestServlet.url + '?q=3', :method => :get }
+    ]
+    Curl::Multi.http(urls, {:pipeline => true, :max_host_connections => 1}) do|easy, code, method|
+      assert_equal 200, code
+      case method
+      when :post
+        assert_match(/POST/, easy.body)
+      when :get
+        assert_match(/GET/, easy.body)
+      when :put
+        assert_match(/PUT/, easy.body)
+      end
+    end
+  end
+
+  # Regression test for issue #267 (2015): ensure that when reusing
+  # easy handles with constrained concurrency, the callback receives
+  # the correct URL for each completed request rather than repeating
+  # the first URL.
+  def test_multi_easy_http_urls_unique_across_max_connects
+    urls = [
+      { :url => TestServlet.url + '?q=1', :method => :get },
+      { :url => TestServlet.url + '?q=2', :method => :get },
+      { :url => TestServlet.url + '?q=3', :method => :get }
+    ]
+
+    [1, 2, 3].each do |max|
+      results = []
+      Curl::Multi.http(urls.dup, {:pipeline => true, :max_connects => max}) do |easy, code, method|
+        assert_equal 200, code
+        assert_equal :get, method
+        results << easy.last_effective_url
+      end
+
+      # Ensure we saw one completion per input URL
+      assert_equal urls.size, results.size, "expected #{urls.size} results with max_connects=#{max}"
+
+      # And that each URL completed exactly once (no accidental reuse/mis-reporting)
+      expected_urls = urls.map { |u| u[:url] }
+      assert_equal expected_urls.to_set, results.to_set, "unexpected URLs for max_connects=#{max}: #{results.inspect}"
+      assert_equal expected_urls.size, results.uniq.size, "duplicate URLs seen for max_connects=#{max}: #{results.inspect}"
+    end
+  end
+
   def test_multi_recieves_500
     m = Curl::Multi.new
     e = Curl::Easy.new("http://127.0.0.1:9129/methods")
